"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parse = void 0;
var orga_1 = require("orga");
var traverse_1 = __importDefault(require("traverse"));
var structured_source_1 = __importDefault(require("structured-source"));
var mapping_1 = require("./mapping");
function parse(org) {
    // TODO: Define return value type.
    var ast = (0, orga_1.parse)(org);
    var src = new structured_source_1.default(org);
    (0, traverse_1.default)(ast).forEach(function (node) {
        if (this.notLeaf) {
            delete node.parent;
            // AST node has type and position
            if (node.type && node.position) {
                node.type = mapping_1.nodeTypes[node.type];
            }
            if (typeof node.type === 'undefined') {
                node.type = 'UNKNOWN';
            }
            // map `range`, `loc` and `raw` to node
            if (typeof node.position === 'object') {
                var position = node.position;
                // Maybe prefer `const { position } = node;`, pure functional. But can't resolve eslint caution...
                // TxtNode's line start with 1
                // TxtNode's column start with 0
                var positionCompensated = {
                    start: { line: position.start.line, column: position.start.column - 1 },
                    end: { line: position.end.line, column: position.end.column - 1 },
                };
                var range = src.locationToRange(positionCompensated);
                node.loc = positionCompensated;
                node.range = range;
                node.raw = org.slice(range[0], range[1]);
                Object.defineProperty(node, "position", {
                    enumerable: false,
                    configurable: false,
                    writable: false,
                    value: position
                });
            }
            // map `url` to Link node
            if (node.type === 'Link' && typeof node.value !== 'undefined') {
                node.url = node.value;
            }
        }
    });
    return ast;
}
exports.parse = parse;
